/*
 * Copyright (c) 2007 Lothar Werzinger
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#include <iostream>
#include <stdexcept>

#include "unittest/unittest.h"

using namespace unittest;

namespace unittest
{
namespace example
{

// create a local testsuite where all the other test suits register.
// this allows us to use the unit test framework in header only mode.
TestSuite localTestSuite("local");


struct Cook
:
  public TestSuite
{
    // test context (fixture) emulating the use of an oven.
    struct Oven
    {
      Oven()
      :
        temperature(142)
      {
        std::cout << "start using Oven." << std::endl;
      }

      ~Oven()
      {
        std::cout << "stop using Oven." << std::endl;
      }

      int temperature;
    };

    // test context (fixture) emulating the use of a pot.
    struct Pot
    {
      Pot()
      {
        std::cout << "start using Pot." << std::endl;
      }

      ~Pot()
      {
        std::cout << "stop using Pot." << std::endl;
      }
    };

    // test context (fixture) emulating the use of a grill.
    struct Grill
    {
      Grill()
      :
        temperature(0),
        coal(5)
      {
        std::cout << "start using Grill." << std::endl;
      }

      ~Grill()
      {
        std::cout << "stop using Grill." << std::endl;
      }

      int temperature;
      int coal;
    };


    struct Burger1
    :
      // we need a grill to BBQ a burger
      public Grill
    {
      void operator()()
      {
        UNITTEST_ASSERT(Equal, 5, coal)("Initial coal amount mismatch.");
        UNITTEST_ASSERT(GreaterEqual, 10, coal)("Not enough coal to start a BBQ.");
        std::cout << "finished Burger." << std::endl;
      }
    };


    struct Burger2
    :
      // we need a grill to BBQ a burger
      public Grill
    {
      void operator()()
      {
        UNITTEST_ASSERT(Equal, 5, coal)("Initial coal amount mismatch.");
        // add coal to grill
        coal += 10;
        UNITTEST_ASSERT(Equal, 15, coal)("New coal amount mismatch.");
        UNITTEST_ASSERT(GreaterEqual, 10, coal)("Not enough coal to start a BBQ.");
        std::cout << "finished Burger." << std::endl;
      }
    };

    struct Soup
    :
      // we need an oven and a pot to cook soup.
      public Oven,
      public Pot
    {
      void operator()()
      {
        UNITTEST_ASSERT(GreaterEqual, 100, temperature)("Temperature too low.");
        UNITTEST_ASSERT(LessEqual, 200, temperature)("Temperature too high.");
        std::cout << "finished Soup." << std::endl;
      }
    };


    Cook()
    {
      add(new Test<Burger1>);
      add(new Test<Burger2>);
      add(new Test<Soup>);
    }
};

// register the Cook test suite with the local main test suite
AutoRegister < Cook > cook(localTestSuite);


struct Bartend
:
  public TestSuite
{
    // test context (fixture) emulating the use of a shaker.
    struct Shaker
    {
      Shaker()
      {
        std::cout << "start using Shaker." << std::endl;
      }

      ~Shaker()
      {
        std::cout << "stop using Shaker." << std::endl;
      }
    };


    struct Martini
    :
      public Shaker
    {
      void operator()()
      {
        throw std::runtime_error("The Martini must be stirred, not shaken!");
      }
    };

    struct Caipirinha
    :
      public Shaker
    {
      void prepare()
      {
        throw std::out_of_range("Not enough lime");
      }

      void operator()()
      {
        UNITTEST_NO_EXCEPTION(
          prepare();
        )
      }
    };

    struct MaiTai
    :
      public Shaker
    {
      void prepare()
      {
        std::cout << "finished MaiTai." << std::endl;
      }

      void operator()()
      {
        // we expect an exception, as we don't have any Rum.
        UNITTEST_EXCEPTION(
          std::out_of_range,
          prepare();
        )
      }
    };

    struct BloodyMary
    :
      // mark this test as unimplemented
      public NotImplemented
    {
      void operator()()
      {
      }
    };

    static void rustyNail(void)
    {
      throw NotImplemented();
    }

    Bartend()
    {
      add(new Test<Martini>);
      add(new Test<Caipirinha>);
      add(new Test<MaiTai>);
      add(new Test<BloodyMary>);
      add(new Test<voidfunc>(&rustyNail, "unittest::example::Bartend::RustyNail"));
    }
};

// register the Bartend test suite with the local main test suite
AutoRegister < Bartend > bartend(localTestSuite);


} /* end of namespace example */
} /* end of namespace unittest */


using namespace unittest::example;


int main(int , char **)
{
  std::stringstream ss;
  // execute the tests (terse)
  TerseExecutor terseExecutor(ss);
  terseExecutor(localTestSuite);
  std::cout << std::endl;
  std::cout << ss.str() << std::endl;

  // execute the tests (verbose)
  VerboseExecutor verboseExecutor(std::cout);
  verboseExecutor(localTestSuite);
  std::cout << std::endl;

  // show all test results (excluding the successful ones)
  ResultVisitor badVisitor(std::cout, false);
  badVisitor(localTestSuite);
  std::cout << std::endl;

  // show all test results (including the successful ones)
  ResultVisitor allVisitor(std::cout, true);
  allVisitor(localTestSuite);
  std::cout << std::endl;

  // show all test results (as XML)
  JunitXmlResultVisitor xmlVisitor(std::cout);
  xmlVisitor(localTestSuite);
  std::cout << std::endl;

  return 0;
} /* end of main */
