#include <exception>
#include <sstream>

#include "unittest/unittest.h"


using namespace unittest;


struct UnitTestExecute
:
  public TestSuite
{
    struct TestOk
    {
      void operator()()
      {
      }
    };
    
    struct TestFail
    {
      void operator()()
      {
        UNITTEST_ASSERT(Equal, true, false)("This test is always failing.");
      }
    };
    
    struct TestStdError
    {
      void operator()()
      {
        throw std::runtime_error("This test is always failing.");
      }
    };
    
    struct TestStringError
    {
      void operator()()
      {
        throw std::string("This test is always failing.");
      }
    };
    
    struct TestCStringError
    {
      void operator()()
      {
        throw "This test is always failing.";
      }
    };
    
    struct TestOtherError
    {
      void operator()()
      {
        throw 42;
      }
    };
    
    struct TestNotImplemented : public NotImplemented
    {
      void operator()()
      {
      }
    };
    
    struct FailingFixture
    {
      FailingFixture()
      {
        throw std::runtime_error("This test is always failing.");
      }
    };
    
    struct TestFailingFixture : public FailingFixture
    {
      void operator()()
      {
      }
    };
    
    struct CreateTests
    {
      CreateTests()
      :
        m_tests("All")
      {
        std::auto_ptr < TestSuite > a_ts;
        
        a_ts = std::auto_ptr < TestSuite >(new TestSuite("functions"));
        a_ts->add(new Test < voidfunc > (CreateTests::testFunctionOk, "ok"));
        a_ts->add(new Test < voidfunc > (CreateTests::testFunctionFail, "fail"));
        a_ts->add(new Test < voidfunc > (CreateTests::testFunctionStdError, "stderror"));
        a_ts->add(new Test < voidfunc > (CreateTests::testFunctionStringError, "stringerror"));
        a_ts->add(new Test < voidfunc > (CreateTests::testFunctionCStringError, "cstringerror"));
        a_ts->add(new Test < voidfunc > (CreateTests::testFunctionOtherError, "othererror"));
        a_ts->add(new Test < voidfunc > (CreateTests::testFunctionNotImplemented, "notimplemented"));
        m_tests.add(a_ts.release());
        
        a_ts = std::auto_ptr < TestSuite >(new TestSuite("functors"));
        a_ts->add(new Test < TestOk > ("ok"));
        a_ts->add(new Test < TestFail > ("fail"));
        a_ts->add(new Test < TestStdError > ("stderror"));
        a_ts->add(new Test < TestStringError > ("stringerror"));
        a_ts->add(new Test < TestCStringError > ("cstringerror"));
        a_ts->add(new Test < TestOtherError > ("othererror"));
        a_ts->add(new Test < TestNotImplemented > ("notimplemented"));
        a_ts->add(new Test < TestFailingFixture > ("failingfixture"));
        m_tests.add(a_ts.release());
      }
      
      ~CreateTests()
      {
      }
      
      static void testFunctionOk(void)
      {
      }
      
      static void testFunctionFail(void)
      {
        UNITTEST_ASSERT(Equal, true, false)("This test is always failing.");
      }
      
      static void testFunctionStdError(void)
      {
        throw std::runtime_error("This test is always failing.");
      }
      
      static void testFunctionStringError(void)
      {
        throw std::string("This test is always failing.");
      }
      
      static void testFunctionCStringError(void)
      {
        throw "This test is always failing.";
      }
      
      static void testFunctionOtherError(void)
      {
        throw 42;
      }
      
      static void testFunctionNotImplemented(void)
      {
        throw NotImplemented();
      }
      
      TestSuite                  m_tests;
    };
    
    struct executeSilent : public CreateTests
    {
      void operator()()
      {
        Executor executor;
        executor(m_tests);
        
        Counter counter;
        counter(m_tests);
        
        UNITTEST_ASSERT(Equal, 15L, counter.total);
        UNITTEST_ASSERT(Equal, 2L, counter.failures);
        UNITTEST_ASSERT(Equal, 9L, counter.errors);
        UNITTEST_ASSERT(Equal, 2L, counter.notImplemented);
      }
    };
    
    struct executeTerse : public CreateTests
    {
      void operator()()
      {
        std::stringstream ss;
        TerseExecutor executor(ss);
        executor(m_tests);
        
        Counter counter;
        counter(m_tests);
        
        UNITTEST_ASSERT(Equal, 15L, counter.total);
        UNITTEST_ASSERT(Equal, 2L, counter.failures);
        UNITTEST_ASSERT(Equal, 9L, counter.errors);
        UNITTEST_ASSERT(Equal, 2L, counter.notImplemented);
        UNITTEST_ASSERT(
          Equal,
          std::string(
            "?\b.?\bF?\bE?\bE?\bE?\bE?\bN?\b.?\bF?\bE?\bE?\bE?\bE?\bN?\bE\n"
            "2 OK, 2 NOT IMPLEMENTED, 2 FAILED, 9 ERROR, 15 total.\n"
          ),
          ss.str()
        );
      }
    };
    
    struct executeVerbose : public CreateTests
    {
      void operator()()
      {
        std::stringstream ss;
        VerboseExecutor executor(ss);
        executor(m_tests);
        
        Counter counter;
        counter(m_tests);
        
        UNITTEST_ASSERT(Equal, 15L, counter.total);
        UNITTEST_ASSERT(Equal, 2L, counter.failures);
        UNITTEST_ASSERT(Equal, 9L, counter.errors);
        UNITTEST_ASSERT(Equal, 2L, counter.notImplemented);
        UNITTEST_ASSERT(
          Equal,
          std::string(
            "ok ...\n"
            "ok OK (0s).\n"
            "fail ...\n"
            "fail FAILED.\n"
            "stderror ...\n"
            "stderror ERROR.\n"
            "stringerror ...\n"
            "stringerror ERROR.\n"
            "cstringerror ...\n"
            "cstringerror ERROR.\n"
            "othererror ...\n"
            "othererror ERROR.\n"
            "notimplemented ...\n"
            "notimplemented NOT IMPLEMENTED.\n"
            "ok ...\n"
            "ok OK (0s).\n"
            "fail ...\n"
            "fail FAILED.\n"
            "stderror ...\n"
            "stderror ERROR.\n"
            "stringerror ...\n"
            "stringerror ERROR.\n"
            "cstringerror ...\n"
            "cstringerror ERROR.\n"
            "othererror ...\n"
            "othererror ERROR.\n"
            "notimplemented ...\n"
            "notimplemented NOT IMPLEMENTED.\n"
            "failingfixture ...\n"
            "failingfixture ERROR.\n"
            "2 OK, 2 NOT IMPLEMENTED, 2 FAILED, 9 ERROR, 15 total.\n"
          ),
          ss.str()
        );
      }
    };
    
    struct list : public CreateTests
    {
      void operator()()
      {
        std::stringstream ss;
        ListVisitor visitor(ss);
        visitor(m_tests);
        
        UNITTEST_ASSERT(
          Equal,
          std::string(
            "All\n"
            "  functions\n"
            "    ok\n"
            "    fail\n"
            "    stderror\n"
            "    stringerror\n"
            "    cstringerror\n"
            "    othererror\n"
            "    notimplemented\n"
            "  functors\n"
            "    ok\n"
            "    fail\n"
            "    stderror\n"
            "    stringerror\n"
            "    cstringerror\n"
            "    othererror\n"
            "    notimplemented\n"
            "    failingfixture\n"
          ),
          ss.str()
        );
      }
    };
    
    struct results : public CreateTests
    {
      void operator()()
      {
        std::stringstream ss;
        Executor executor;
        executor(m_tests);
        
        ResultVisitor resultVisitor(ss);
        resultVisitor(m_tests);
        
        UNITTEST_ASSERT(
          Equal,
          std::string(
            "All\n"
            "  functions\n"
            "    fail FAILED\n"
            "      false == true FAILED.\n"
            "      0 == 1 FAILED. This test is always failing.\n"
            "      file:src/unittest/test/unittestexecute.cpp, line:123\n"
            "    stderror ERROR\n"
            "      An exception of type std::runtime_error was thrown.\n"
            "      This test is always failing.\n"
            "    stringerror ERROR\n"
            "      This test is always failing.\n"
            "    cstringerror ERROR\n"
            "      This test is always failing.\n"
            "    othererror ERROR\n"
            "      Unknown C++ exception\n"
            "    notimplemented NOT IMPLEMENTED\n"
            "  functors\n"
            "    fail FAILED\n"
            "      false == true FAILED.\n"
            "      0 == 1 FAILED. This test is always failing.\n"
            "      file:src/unittest/test/unittestexecute.cpp, line:25\n"
            "    stderror ERROR\n"
            "      An exception of type std::runtime_error was thrown.\n"
            "      This test is always failing.\n"
            "    stringerror ERROR\n"
            "      This test is always failing.\n"
            "    cstringerror ERROR\n"
            "      This test is always failing.\n"
            "    othererror ERROR\n"
            "      Unknown C++ exception\n"
            "    notimplemented NOT IMPLEMENTED\n"
            "    failingfixture ERROR\n"
            "      An exception of type std::runtime_error was thrown.\n"
            "      Test setup (Constructor) failed. This test is always failing.\n"
          ),
          ss.str()
        );
      }
    };
    
    struct xmlResults : public CreateTests
    {
      void operator()()
      {
        std::stringstream ss;
        Executor executor;
        executor(m_tests);
        
        JunitXmlResultVisitor resultVisitor(ss);
        resultVisitor(m_tests);
        
        UNITTEST_ASSERT(
          Equal,
          std::string(
            "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
            "<testsuite name=\"All\" tests=\"15\" failures=\"2\" errors=\"11\" errors_real=\"2\" errors_notimplemented=\"2\" time=\"0\">\n"
            "  <testsuite name=\"functions\" tests=\"7\" failures=\"1\" errors=\"5\" errors_real=\"1\" errors_notimplemented=\"1\" time=\"0\">\n"
            "    <testcase name=\"ok\" time=\"0\"/>\n"
            "    <testcase name=\"fail\" time=\"0\">\n"
            "      <failure file=\"src/unittest/test/unittestexecute.cpp\" line=\"123\" type=\"unittest::Exception\" message=\"false == true FAILED.\">0 == 1 FAILED. This test is always failing.</failure>\n"
            "    </testcase>\n"
            "    <testcase name=\"stderror\" time=\"0\">\n"
            "      <error type=\"std::runtime_error\" message=\"An exception of type std::runtime_error was thrown.\">This test is always failing.</error>\n"
            "    </testcase>\n"
            "    <testcase name=\"stringerror\" time=\"0\">\n"
            "      <error type=\"std::string\" message=\"This test is always failing.\"></error>\n"
            "    </testcase>\n"
            "    <testcase name=\"cstringerror\" time=\"0\">\n"
            "      <error type=\"char const*\" message=\"This test is always failing.\"></error>\n"
            "    </testcase>\n"
            "    <testcase name=\"othererror\" time=\"0\">\n"
            "      <error type=\"Unknown C++ exception\" message=\"Unknown C++ exception\"></error>\n"
            "    </testcase>\n"
            "    <testcase name=\"notimplemented\" time=\"0\">\n"
            "      <error type=\"unittest::NotImplemented\" message=\"Test has not been implemented (yet).\">Test has not been implemented (yet).</error>\n"
            "    </testcase>\n"
            "  </testsuite>\n"
            "  <testsuite name=\"functors\" tests=\"8\" failures=\"1\" errors=\"6\" errors_real=\"1\" errors_notimplemented=\"1\" time=\"0\">\n"
            "    <testcase name=\"ok\" time=\"0\"/>\n"
            "    <testcase name=\"fail\" time=\"0\">\n"
            "      <failure file=\"src/unittest/test/unittestexecute.cpp\" line=\"25\" type=\"unittest::Exception\" message=\"false == true FAILED.\">0 == 1 FAILED. This test is always failing.</failure>\n"
            "    </testcase>\n"
            "    <testcase name=\"stderror\" time=\"0\">\n"
            "      <error type=\"std::runtime_error\" message=\"An exception of type std::runtime_error was thrown.\">This test is always failing.</error>\n"
            "    </testcase>\n"
            "    <testcase name=\"stringerror\" time=\"0\">\n"
            "      <error type=\"std::string\" message=\"This test is always failing.\"></error>\n"
            "    </testcase>\n"
            "    <testcase name=\"cstringerror\" time=\"0\">\n"
            "      <error type=\"char const*\" message=\"This test is always failing.\"></error>\n"
            "    </testcase>\n"
            "    <testcase name=\"othererror\" time=\"0\">\n"
            "      <error type=\"Unknown C++ exception\" message=\"Unknown C++ exception\"></error>\n"
            "    </testcase>\n"
            "    <testcase name=\"notimplemented\" time=\"0\">\n"
            "      <error type=\"unittest::NotImplemented\" message=\"Test has not been implemented (yet).\">Test has not been implemented (yet).</error>\n"
            "    </testcase>\n"
            "    <testcase name=\"failingfixture\" time=\"0\">\n"
            "      <error type=\"std::runtime_error\" message=\"An exception of type std::runtime_error was thrown.\">Test setup (Constructor) failed. This test is always failing.</error>\n"
            "    </testcase>\n"
            "  </testsuite>\n"
            "</testsuite>\n"
          ),
          ss.str()
        );
      }
    };
    
    
    UnitTestExecute(){
      add(new Test<executeSilent>);
      add(new Test<executeTerse>);
      add(new Test<executeVerbose>);
      add(new Test<list>);
      add(new Test<results>);
      add(new Test<xmlResults>);
    }
};

static AutoRegister<UnitTestExecute> ts;
