/*
 * Copyright (c) 2007 Lothar Werzinger
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef unittest_unittest_h
#define unittest_unittest_h


/*! \mainpage Unit test framework for C++.
 *
 * \section intro_sec Introduction
 *
 * This framework strives to be a simple and small unit test framework for C++.
 * It's goal is to have a C++-ish look and feel.
 * Therefore it uses many C++ features like templates and makes heavy use
 * of the STL.
 * As the code is meant for testing it was a deliberate design choice
 * to make all classes public (structs).
 * 
 * \b Disclaimer: If you don't like this, this unit test framework is simply
 * not for you.
 * 
 * Unlike other unit test frameworks it does NOT use \em setup and \em teardown
 * methods and instead provides the user with a very flexible way to create
 * unit test contexts (or fixtures) where the Constructor represents the
 * \em setup method that other frameworks provide and the Destructor represents
 * the \em teardown method.
 * This is the Resource Acquisition Is Initialization design pattern and
 * should be more familiar to C++ users than the Java-ish use of \em setup and 
 * \em teardown methods.
 * The framework also tries to keep the number of macros to an absolute minimum.
 * However some macros are necessary to present the user with the location of the
 * problem if an assert fails.
 * 
 * The recommended way to create tests is to use functors (function objects),
 * but plain functions or static methods are supported, too.
 * However most of the goodies like contexts (fixtures) are only available for
 * functors. The benefit of this approach is that the test has full access
 * to the members of the contexts.
 *
 * \section install_sec Installation
 *
 * \subsection header_subsec Header only use:
 * If the global test suite (\ref unittest::TestSuite::instance) is not going to be used
 * no installation is neccessary.
 * Everything needed is contained in the header file.
 * 
 * See example.cpp to see how this is done. \see example.cpp
 * 
 * \subsection library_subsec Library use:
 * If the global test suite (\ref unittest::TestSuite::instance) is to be used a library
 * contaning the instance needs to be build and linked to the executable.
 * This allows individual tests to be built as individual libraries and test
 * executables can the link all test libraries together to form a test suite.
 * The tests in the individual libraries register themselves with the
 * global test suite (\ref unittest::TestSuite::instance).
 * 
 * The provided test runner makes use of the
 * global test suite (\ref unittest::TestSuite::instance). \see testrunner.cpp
 * 
 */

/* \example testrunner.cpp
 * This a general purpose console test runner.
 * It show how to use the framework in \ref library_subsec mode.
 */

/* \example example.cpp
 * This is an example of how to use the Unit test framework.
 * The example uses the framework in \ref header_subsec mode.
 * 
 * If you compile the example and run it you will see the
 * following output:
 * \verbatim
start using Grill.
stop using Grill.
start using Grill.
finished Burger.
stop using Grill.
start using Oven.
start using Pot.
finished Soup.
stop using Pot.
stop using Oven.
start using Shaker.
stop using Shaker.
start using Shaker.
stop using Shaker.
start using Shaker.
finished MaiTai.
stop using Shaker.

F..EFFNN
2 OK, 2 NOT IMPLEMENTED, 3 FAILED, 1 ERROR, 8 total.

unittest::example::Cook::Burger1 ...
start using Grill.
stop using Grill.
unittest::example::Cook::Burger1 FAILED.
unittest::example::Cook::Burger2 ...
start using Grill.
finished Burger.
stop using Grill.
unittest::example::Cook::Burger2 OK.
unittest::example::Cook::Soup ...
start using Oven.
start using Pot.
finished Soup.
stop using Pot.
stop using Oven.
unittest::example::Cook::Soup OK.
unittest::example::Bartend::Martini ...
start using Shaker.
stop using Shaker.
unittest::example::Bartend::Martini ERROR.
unittest::example::Bartend::Caipirinha ...
start using Shaker.
stop using Shaker.
unittest::example::Bartend::Caipirinha FAILED.
unittest::example::Bartend::MaiTai ...
start using Shaker.
finished MaiTai.
stop using Shaker.
unittest::example::Bartend::MaiTai FAILED.
unittest::example::Bartend::BloodyMary ...
unittest::example::Bartend::BloodyMary NOT IMPLEMENTED.
unittest::example::Bartend::RustyNail ...
unittest::example::Bartend::RustyNail NOT IMPLEMENTED.
2 OK, 2 NOT IMPLEMENTED, 3 FAILED, 1 ERROR, 8 total.

local
  unittest::example::Cook
    unittest::example::Cook::Burger1 FAILED
      coal >= 10 FAILED.
      5 >= 10 FAILED. Not enough coal to start a BBQ.
      file:src/unittest/example/example.cpp, line:104
  unittest::example::Bartend
    unittest::example::Bartend::Martini ERROR
      An exception of type std::runtime_error was thrown.
      The Martini must be stirred, not shaken!
    unittest::example::Bartend::Caipirinha FAILED
      Expected no exception to be thrown.
      Caught std::out_of_range instead (Not enough lime).
      file:src/unittest/example/example.cpp, line:194
    unittest::example::Bartend::MaiTai FAILED
      Expected exception std::out_of_range to be thrown.
      No exception was thrown.
      file:src/unittest/example/example.cpp, line:213
    unittest::example::Bartend::BloodyMary NOT IMPLEMENTED
    unittest::example::Bartend::RustyNail NOT IMPLEMENTED

local
  unittest::example::Cook
    unittest::example::Cook::Burger1 FAILED
      coal >= 10 FAILED.
      5 >= 10 FAILED. Not enough coal to start a BBQ.
      file:src/unittest/example/example.cpp, line:104
    unittest::example::Cook::Burger2 OK
    unittest::example::Cook::Soup OK
  unittest::example::Bartend
    unittest::example::Bartend::Martini ERROR
      An exception of type std::runtime_error was thrown.
      The Martini must be stirred, not shaken!
    unittest::example::Bartend::Caipirinha FAILED
      Expected no exception to be thrown.
      Caught std::out_of_range instead (Not enough lime).
      file:src/unittest/example/example.cpp, line:194
    unittest::example::Bartend::MaiTai FAILED
      Expected exception std::out_of_range to be thrown.
      No exception was thrown.
      file:src/unittest/example/example.cpp, line:213
    unittest::example::Bartend::BloodyMary NOT IMPLEMENTED
    unittest::example::Bartend::RustyNail NOT IMPLEMENTED

<?xml version="1.0" encoding="UTF-8"?>
<testsuite name="local" tests="8" errors="1" failures="3">
  <testsuite name="unittest::example::Cook" tests="3" errors="0" failures="1">
    <testcase name="unittest::example::Cook::Burger1">
      <failure type="unittest::Exception" file="src/unittest/example/example.cpp" line="104" message="coal &gt;= 10 FAILED.">5 &gt;= 10 FAILED. Not enough coal to start a BBQ.</failure>
    </testcase>
    <testcase name="unittest::example::Cook::Burger2"/>
    <testcase name="unittest::example::Cook::Soup"/>
  </testsuite>
  <testsuite name="unittest::example::Bartend" tests="5" errors="1" failures="2">
    <testcase name="unittest::example::Bartend::Martini">
      <error type="std::runtime_error" message="An exception of type std::runtime_error was thrown.">The Martini must be stirred, not shaken!</error>
    </testcase>
    <testcase name="unittest::example::Bartend::Caipirinha">
      <failure type="unittest::Exception" file="src/unittest/example/example.cpp" line="194" message="Expected no exception to be thrown.">Caught std::out_of_range instead (Not enough lime).</failure>
    </testcase>
    <testcase name="unittest::example::Bartend::MaiTai">
      <failure type="unittest::Exception" file="src/unittest/example/example.cpp" line="213" message="Expected exception std::out_of_range to be thrown.">No exception was thrown.</failure>
    </testcase>
    <testcase name="unittest::example::Bartend::BloodyMary">
      <error type="" message="Test has not been implemented (yet)."></error>
    </testcase>
    <testcase name="unittest::example::Bartend::RustyNail">
      <error type="" message="Test has not been implemented (yet)."></error>
    </testcase>
  </testsuite>
</testsuite>
   \endverbatim 
 */


#ifdef __GNUC__
#include <cxxabi.h>
#endif

#include <cmath>
#include <ctime>
#include <list>
#include <vector>
#include <algorithm>
#include <iostream>
#include <sstream>
#include <stdexcept>

#include <tr1/memory>


namespace unittest
{
// forward declarations
struct Exception;
struct TestBase;
struct TestSuite;
} /* end of namespace unittest */


namespace std
{
// forward declarations
template < class charT, class traits >
std::basic_ostream < charT, traits > & operator << (
  std::basic_ostream < charT, traits > & ostream,
  unittest::Exception const & ex
);
} /* end of namespace std */


namespace unittest
{


/** Guard class to free memory obtained with ::malloc
 * It works like std::auto_ptr just for malloced memory.
 */
class malloc_auto_ptr
{
  public:
    /** Create a malloc guard.
     */
    malloc_auto_ptr()
    :
      buffer(0)
    {
    }
    
    /** Create a malloc guard.
     * @param newbuffer   The newly malloced buffer.
     */
    malloc_auto_ptr(void * newbuffer)
    :
      buffer(newbuffer)
    {
    }
    
    /** Create a malloc guard.
     * @param copy   Another malloc guard to take the buffer from. 
     */
    malloc_auto_ptr(malloc_auto_ptr & copy)
    :
      buffer(0)
    {
      buffer = copy.release();
    }
    
    /** Destroy a malloc guard.
     * If the guard holds a buffer it will be freed.
     */
    ~malloc_auto_ptr()
    {
      reset();
    }
    
    /** Assign a malloc guard.
     * @param copy   Another malloc guard to take the buffer from. 
     */
    malloc_auto_ptr & operator = (malloc_auto_ptr & copy)
    {
      reset();
      buffer = copy.release();
      return *this;
    }
    
    /** Get the underlying buffer.
     * @return  The underlying buffer.
     */
    void * get() const
    {
      return buffer;
    }
  
    /** Release the underlying buffer.
     * @return  The underlying buffer.
     * The recipient of the buffer is responsable to free the buffer.
     */
    void * release()
    {
      void * oldbuffer = buffer;
      buffer = 0;
      return oldbuffer;
    }
    
    /** Free the underlying buffer.
     */
    void reset()
    {
      if(buffer)
        ::free(buffer);
      buffer = 0;
    }
  
  private:
    malloc_auto_ptr(malloc_auto_ptr const & copy); // not allowed
    malloc_auto_ptr & operator = (malloc_auto_ptr const & copy); // not allowed
  
  private:
    void * buffer;
};


/** Get the type name of a type.
 * This method demangles the type name when used with g++
 */
inline std::string typeName(std::type_info const & typeInfo)
{
  std::string result = typeInfo.name();
#ifdef __GNUC__
  int status = -1;
  char * demangledName = abi::__cxa_demangle(result.c_str(), 0, 0, &status);
  malloc_auto_ptr buffer(demangledName);
  if(status == 0)
  {
    result = std::string(demangledName);
  }
#endif
  return result;
} /* end of typeName ( ) */


/** The abstract base class for the visitors.
 * This class implements the visitor pattern by overloading the
 * function call operator instead of using "visit" methods.
 */
struct Visitor
{
  /** Destroy the \ref Visitor.
   */
  virtual ~Visitor()
  {
  }
  
  /** Method called when visiting a test.
   * @param test  The test to visit.
   */
  virtual void operator()(TestBase & test) const = 0;
  
  /** Method called when visiting a test suite.
   */
  virtual void operator()(TestSuite & suite) const = 0;
};


/** The exception used to store information about the
 * failure of a test.
 */
struct Exception
{
  /** Create an \ref Exception without location information.
   * @param message     The brief problem message.
   * @param details     The problem details.
   */
  Exception(
    std::string const & message = std::string(),
    std::string const & details = std::string()
  )
  :
    message(message),
    details(details),
    file(),
    line(0)
  {
  }
  
    /** Create an \ref Exception with location information.
     * @param message     The brief problem message.
     * @param details     The problem details.
     * @param file        The file name.
     * @param line        The line number.
     */
  Exception(
    std::string const & file,
    long line,
    std::string const & message = std::string(),
    std::string const & details = std::string()
  )
  :
    message(message),
    details(details),
    file(file),
    line(line)
  {
  }
  
  /** The brief problem message.
   */
  std::string message;
  /** The problem details.
   */
  std::string details;
  /** The file name.
   */
  std::string file;
  /** The line number.
   */
  long line;
};


/** Test context (fixture) to indicate that
 * the test inheriting from this class
 * has not been implemented (yet).
 */
struct NotImplemented
:
  public std::runtime_error
{
  /** Throw an exception to signal that a test is unimplemented.
   */
  NotImplemented()
  :
    std::runtime_error(
      "Test has not been implemented (yet)."
    )
  {
    throw *this;
  }
};


/** The base class to store information about tests.
 * This class has two purposes. First to be a base class to tests
 * and test suits, so that they can be stored arbitrarily inside a suite.
 * Second to collect some information about the test (like status and
 * additional information like messages. It also allows the user to give a
 * user defined name to the test(suite).
 */
struct TestBase
{
  /// The test status.
  enum TestStatus
  {
    /// The test was successful.
    eSuccess,
    /// The test is not implemented.
    eNotImplemented,
    /// The test failed.
    eFailed,
    /// The test had an error (unexpected exception).
    eError
  };
  
  /** Create a \ref TestBase.
   * @param name   Optional user provided name for the test.
   */
  TestBase(std::string name = "")
  :
    username(name),
    error(),
    exception(),
    status(eSuccess),
    duration(0.0)
  {
  }
  
  /** Destroy a \ref TestBase.
   */
  virtual ~TestBase()
  {
  }
  
  /** Method to accept visitors.
   * @param visitor  The visitor to accept.
   */
  virtual void operator()(Visitor const & visitor) = 0;
  
  /** Virtual method to execute the test.
   */
  virtual void operator()(void) = 0;
  
  /** Get the name of the test.
   * This method returns the type name if the user did not
   * specify an user defined name.
   */
  virtual std::string name() const
  {
    if(username.empty())
      return typeName(typeid(*this));
    else
      return username;
  }
  
  /** The optional user define name
   */
  std::string username;
  /** The error as a string.
   */
  std::string error;
  /** The \ref Exception.
   */
  Exception exception;
  /** The \ref TestStatus.
   */
  TestStatus status;
  /** The duration of the test.*/
  double duration;
};


/** Template to create tests from functors (function objects).
 * This is the preferred way to create tests as functors are much
 * more powerful than plain functions.
 */
template <typename T> struct Test
:
  public TestBase
{
  /** Typedef with the "real" type of the test.
   */
  typedef T test_type;
  
  /** Create a test.
   * If no name is given the typename of the test tempate parameter is used
   * as the test name.
   * @param name   Optional test name.
   */
  Test(std::string name = "")
  :
    TestBase(name)
  {
  }
  
  /** Destroy a test.
   */
  virtual ~Test()
  {
  }
  
  /** Get the name of the test.
   * This method returns the type name of the test if the user did not
   * specify an user defined name.
   */
  virtual std::string name() const
  {
    if(username.empty())
      return typeName(typeid(test_type));
    else
      return username;
  }
  
  /** Execute the test.
   */
  void operator()()
  {
    error = "";
    exception = Exception();
    status = eSuccess;
    bool constructor = true;
    try
    {
      std::clock_t start = std::clock();
      T t;
      constructor = false;
      t();
      duration = (clock() - start);
      duration /= CLOCKS_PER_SEC;
    }
    catch(Exception const & ex)
    {
      status = eFailed;
      error = typeName(typeid(ex));
      exception = ex;
    }
    catch(NotImplemented const & ex)
    {
      status = eNotImplemented;
      error = "";
      exception.message = "";
      exception.details = "";
    }
    catch(std::exception const & ex)
    {
      status = eError;
      error = typeName(typeid(ex));
      std::ostringstream ss;
      ss << "An exception of type " << error << " was thrown.";
      exception.message = ss.str();
      exception.details = ex.what();
    }
    catch(std::string const & ex)
    {
      status = eError;
      error = typeName(typeid(ex));
      exception.message = ex;
    }
    catch(char const * const ex)
    {
      status = eError;
      error = typeName(typeid(ex));
      exception.message = ex;
    }
    catch(...)
    {
      status = eError;
      error = "Unknown C++ exception";
      exception.message = error;
    }
    
    if(status && constructor)
    {
      if(status != eNotImplemented)
      {
        exception.details = "Test setup (Constructor) failed. " + exception.details;
      }
    }
  }
  
  /** Method to accept visitors.
   * @param visitor  The visitor to accept.
   */
  void operator()(Visitor const & visitor)
  {
    visitor(*this);
  }
};

/** Typedef for a function based test.
 */
typedef void (*voidfunc)(void);

/** Template to create tests from plain functions or static methods.
 */
template < > struct Test < voidfunc > 
:
  public TestBase
{
  /** Create a test.
   * As plain functions or static methods don't provide a means of
   * automatically derive the name a user defined name must be given.
   * @param func   The test function.
   * @param name   Mandatory test name.
   */
  Test(
    voidfunc func,
    std::string name
  )
  :
    TestBase(name),
    func(func)
  {
  }
  
  /** Destroy a test.
   */
  virtual ~Test()
  {
  }
  
  /** Execute the test.
   */
  void operator()()
  {
    error = "";
    exception = Exception();
    status = eSuccess;
    try
    {
      func();
    }
    catch(Exception const & ex)
    {
      status = eFailed;
      error = typeName(typeid(ex));
      exception = ex;
    }
    catch(NotImplemented const & ex)
    {
      status = eNotImplemented;
      error = "";
      exception.message = "";
      exception.details = "";
    }
    catch(std::exception const & ex)
    {
      status = eError;
      error = typeName(typeid(ex));
      std::ostringstream ss;
      ss << "An exception of type " << error << " was thrown.";
      exception.message = ss.str();
      exception.details = ex.what();
    }
    catch(std::string const & ex)
    {
      status = eError;
      error = typeName(typeid(ex));
      exception.message = ex;
    }
    catch(char const * const ex)
    {
      status = eError;
      error = typeName(typeid(ex));
      exception.message = ex;
    }
    catch(...)
    {
      status = eError;
      error = "Unknown C++ exception";
      exception.message = error;
    }
  }
  
  /** Method to accept visitors.
   * @param visitor  The visitor to accept.
   */
  void operator()(Visitor const & visitor)
  {
    visitor(*this);
  }
  
  /** The test function.
   */
  voidfunc func;
};


/** A test suite is a collection of tests or other test suites.
 * 
 */
struct TestSuite
:
  public TestBase
{
  /** Typedef for the shared \ref TestBase pointer.
   */
  typedef std::tr1::shared_ptr < TestBase > TestBasePtr;
  /** Typedef for the container for \ref TestBase objects.
   */
  typedef std::vector < TestBasePtr > TestContainer;
  
  /** Create the test suite.
   * @param name   Optional test name.
   */
  TestSuite(std::string name = "")
  :
    TestBase(name)
  {
  }
  
  /** Destroy a test suite.
   */
  virtual ~TestSuite()
  {
  }
  
  /** Method to accept visitors.
   * @param visitor  The visitor to accept.
   */
  void operator()(Visitor const & visitor)
  {
    visitor(*this);
  }
  
  /** Virtual method to execute the test.
   */
  virtual void operator()(void)
  {
    //nop
  }
  
  /** Add a test to the suite.
   * @param test   The test to add.
   */
  void add(TestBasePtr test)
  {
    tests.push_back(test);
  }
  
  /** Add a test to the suite.
   * @param test   The test to add.
   * The test needs to be allocated with new and the suite
   * is taking over ownership to that test.
   */
  void add(TestBase * test)
  {
    tests.push_back(TestBasePtr(test));
  }
  
  /** The global test suite instance.
   * @return  The global test suite instance.
   */
  static TestSuite & instance(void);
  
  /** The container of tests.
   */
  TestContainer tests;
};


/** A visitor that counts the successful and failed tests.
 */
struct Counter
:
  public Visitor
{
  /** Create a counter.
   */
  Counter()
  :
    total(0),
    notImplemented(0),
    failures(0),
    errors(0),
    duration(0.0)
  {
  }
  
  /** Method called when visiting a test.
   * @param test  The test to visit.
   */
  void operator()(TestBase & test) const
  {
    ++total;
    if(test.status == TestBase::eNotImplemented)
      ++notImplemented;
    else if(test.status == TestBase::eFailed)
      ++failures;
    else if(test.status == TestBase::eError)
      ++errors;
    duration += test.duration;
  }
  
  /** Method called when visiting a test suite.
   * @param suite  The test suite to visit.
   */
  void operator()(TestSuite & suite) const
  {
    TestSuite::TestContainer::iterator it, end;
    for (
      it = suite.tests.begin(),
      end = suite.tests.end();
      it != end;
      ++it
    )
    {
      TestBase & test = *(*it);
      test(*this);
    } /* end of for */
  }
  
  /** Print summary information.
   * @param ostream   The stream to print the information to.
   */
  void summary( std::ostream & ostream )
  {
    ostream << (total - notImplemented - failures - errors) << " OK, ";
    ostream << notImplemented << " NOT IMPLEMENTED, ";
    ostream << failures << " FAILED, ";
    ostream << errors << " ERROR, ";
    ostream << total << " total.";
    ostream << std::endl;
  }
  
  /** The total count of all tests.
   */
  mutable long total;
  /** The count of all tests that failed.
   */
  mutable long notImplemented;
  /** The count of all tests that failed.
   */
  mutable long failures;
  /** The count of all tests that failed with an error.
   */
  mutable long errors;
  /** The duration of all tests.
   */
  mutable double duration;
};


/** A test suite that let's the user select which tests to run.
 */
struct SelectSuite
:
  public TestSuite
{
  /** Create the test suite.
   * @param name   Optional test name.
   */
  SelectSuite ( std::string const & name = "" )
  :
    TestSuite(name)
  {
  }
  
  /** Create the test suite.
   * @param name          Test name.
   * @param includeList   Tests to include.
   *                      If empty all tests get included.
   * @param excludeList   Tests to exclude.
   */
  SelectSuite (
    std::string const & name,
    std::vector < std::string > const & includeList,
    std::vector < std::string > const & excludeList
  )
  :
    TestSuite(name),
    includeList(includeList),
    excludeList(excludeList)
  {
  }
    
  /** Add a test name to the include list.
   * @param  testname   The name of the test to include.
   */
  void include ( std::string const & testname )
  {
    includeList.push_back(testname);
  }
  
  /** Add a test name to the exlude list.
   * @param  testname   The name of the test to exclude.
   */
  void exclude ( std::string const & testname )
  {
    excludeList.push_back(testname);
  }
  
  /** Process the selection.
   * A test suite passed will be visited to find any test(suit)s
   * that match the selection. If a match occurs the test(suite)
   * is copied to this suite.
   * @param  visitor   The testsuite to process.
   */
  void operator()(Visitor const & visitor)
  {
    visitor(*this);
  }
  
  using TestSuite::operator();
  
  /** Method called when visiting a test suite.
   * A test suite passed will be visited to find any test(suit)s
   * that match the selection. If a match occurs the test(suite)
   * is copied to this suite.
   * @param  visitor   The testsuite to process.
   */
  virtual void operator()(TestSuite & suite) const
  {
    TestSuite::TestContainer::iterator it, end;
    std::vector < std::string > emptyIncludeList;
    std::list < std::string > missedIncludeList(includeList.begin(), includeList.end());
    
    for (
      it = suite.tests.begin(),
      end = suite.tests.end();
      it != end;
      ++it
    )
    {
      TestBase & test = *(*it);
      if(std::find(
           includeList.begin(),
           includeList.end(),
           test.name()
         ) != includeList.end()
      )
      {
        missedIncludeList.remove(test.name());
      }
    }
    
    for (
      it = suite.tests.begin(),
      end = suite.tests.end();
      it != end;
      ++it
    )
    {
      TestBase & test = *(*it);
      
      if(std::find(
           excludeList.begin(),
           excludeList.end(),
           test.name()
         ) != excludeList.end()
      )
      {
        // skip excluded test
        continue;
      } /* end of if */
      
      if (
        includeList.empty() ||
        std::find(
          includeList.begin(),
          includeList.end(),
          test.name()
        ) != includeList.end()
      )
      {
        if (dynamic_cast<TestSuite *>(&test))
        {
          SelectSuite * ss = new SelectSuite(
            test.name(),
            emptyIncludeList,
            excludeList
          );
          (*ss)(dynamic_cast<TestSuite &>(test));
          
          Counter counter;
          counter(*ss);
          
          if(counter.total > 0)
            const_cast < SelectSuite & > (*this).add(ss);
        } /* end of if */
        else
        {
          const_cast < SelectSuite & > (*this).add(*it);
        } /* end of else */
      } /* end of if */
      
      // handle the missed includes
      if (
        !missedIncludeList.empty() &&
        dynamic_cast<TestSuite *>(&test)
      )
      {
        SelectSuite * ss = new SelectSuite(
          test.name(),
          std::vector < std::string >(
            missedIncludeList.begin(),
            missedIncludeList.end()
          ),
          excludeList
        );
        (*ss)(dynamic_cast<TestSuite &>(test));
        
        Counter counter;
        counter(*ss);
        
        if(counter.total > 0)
          const_cast < SelectSuite & > (*this).add(ss);
      } /* end of if */
    } /* end of for */
  }
  
  /** The selected test names.
   */
  std::vector < std::string > includeList;
  std::vector < std::string > excludeList;
};


/** Autoregister a test(suite).
 */
template < typename T > struct AutoRegister
{
  /** Autoregister a test(suite).
   * Registers with the TestSuite::instance.
   */
  AutoRegister()
  {
    TestSuite::instance().add(TestSuite::TestBasePtr(new T));
  }
  
  
  
  /** Autoregister a test(suite).
   * Registers with the given test suite.
   * @param suite   The test suite to register with.
   */
  AutoRegister(TestSuite & suite)
  {
    suite.add(TestSuite::TestBasePtr(new T));
  }
};


/** Executes tests silently.
 */
struct Executor
:
  public Visitor
{
  /** Method called when visiting a test.
   * @param test  The test to visit.
   */
  void operator()(TestBase & test) const
  {
    test();
  }
  
  /** Method called when visiting a test suite.
   * @param suite  The test suite to visit.
   */
  void operator()(TestSuite & suite) const
  {
    TestSuite::TestContainer::iterator it, end;
    for (
      it = suite.tests.begin(),
      end = suite.tests.end();
      it != end;
      ++it
    )
    {
      TestBase & test = *(*it);
      test(*this);
    } /* end of for */
  }
};


/** Helper class to build visitors that print to ostreams.
 */
struct OstreamVisitor
:
  public Visitor
{
  /** Create the ostream visitor.
   * @param ostream    The stream to write to.
   */
  OstreamVisitor(std::ostream & ostream)
  :
    depth(0),
    ostream(ostream)
  {
  }
  
  /** Indent the output according to the current nesting level.
   * Each increment of depth indents 2 spaces.
   */
  void indent(void) const
  {
    for (int var = 0; var < depth; ++var)
    {
      ostream << "  ";
    } /* end of for */
  }
  
  /** The depth to use for the \ref indent method.
   * Each increment of depth indents 2 spaces.
   */
  mutable int depth;
  /** The ostream to write to.
   */
  std::ostream & ostream;
};


/** Executes tests and prints terse information.
 * This executor prints terse information while executing the tests.
 */
struct TerseExecutor
:
  public virtual OstreamVisitor,
  public virtual Executor
{
  /** Create the executor.
   * @param  ostream    The stream to print to.
   */
  TerseExecutor ( std::ostream & ostream )
  :
    OstreamVisitor(ostream)
  {
  }
  
  /** Method called when visiting a test.
   * @param test  The test to visit.
   */
  void operator()(TestBase & test) const
  {
    ostream << "?" << std::flush;
    Executor::operator()(test);
    ostream << "\b";
    switch(test.status)
    {
      case TestBase::eSuccess:
        ostream << ".";
        break;
      case TestBase::eNotImplemented:
        ostream << "N";
        break;
      case TestBase::eFailed:
        ostream << "F";
        break;
      case TestBase::eError:
        ostream << "E";
        break;
    }
    ostream << std::flush;
  }
  
  /** Method called when visiting a test suite.
   * @param suite  The test suite to visit.
   */
  void operator()(TestSuite & suite) const
  {
    ++depth;
    Executor::operator()(suite);
    --depth;
    if(depth == 0)
    {
      ostream << std::endl;
      Counter counter;
      counter(suite);
      counter.summary(ostream);
    }
  }
};


/** Executes tests and prints verbose information.
 * This executor prints verbose information while executing the tests.
 */
struct VerboseExecutor
:
  public virtual OstreamVisitor,
  public virtual Executor
{
  /** Create the executor.
   * @param  ostream    The stream to print to.
   */
  VerboseExecutor(std::ostream & ostream)
  :
    OstreamVisitor(ostream)
  {
  }
  
  /** Method called when visiting a test.
   * @param test  The test to visit.
   */
  void operator()(TestBase & test) const
  {
    ostream << test.name() << " ..." << std::endl;
    Executor::operator()(test);
    ostream << test.name() << " ";
    switch(test.status)
    {
      case TestBase::eSuccess:
        ostream << "OK (" << test.duration << "s)";
        break;
      case TestBase::eNotImplemented:
        ostream << "NOT IMPLEMENTED";
        break;
      case TestBase::eFailed:
        ostream << "FAILED";
        break;
      case TestBase::eError:
        ostream << "ERROR";
        break;
    }
    ostream << "." << std::endl;
  }
  
  /** Method called when visiting a test suite.
   * @param suite  The test suite to visit.
   */
  void operator()(TestSuite & suite) const
  {
    ++depth;
    Executor::operator()(suite);
    --depth;
    if(depth == 0)
    {
      Counter counter;
      counter(suite);
      counter.summary(ostream);
    }
  }
};


/** Print the names of all tests.
 */
struct ListVisitor
:
  public OstreamVisitor
{
  /** Create the list visitor.
   * @param ostream    The stream to write to.
   */
  ListVisitor(std::ostream & ostream)
  :
    OstreamVisitor(ostream)
  {
  }
  
  /** Method called when visiting a test.
   * @param test  The test to visit.
   */
  void operator()(TestBase & test) const
  {
    indent();
    ostream << test.name();
    ostream << std::endl;
  }
  
  /** Method called when visiting a test suite.
   * @param suite  The test suite to visit.
   */
  void operator()(TestSuite & suite) const
  {
    indent();
    ostream << suite.name() << std::endl;
    ++depth;
    
    TestSuite::TestContainer::iterator it, end;
    for (
      it = suite.tests.begin(),
      end = suite.tests.end();
      it != end;
      ++it
    )
    {
      TestBase & test = *(*it);
      test(*this);
    } /* end of for */

    --depth;
  }
};


/** Print the results of all tests.
 */
struct ResultVisitor
:
  public OstreamVisitor
{
  /** Create the result visitor.
   * @param ostream    The stream to write to.
   * @param printGood  Print the successful tests.
   * If set to true all tests will be printed.
   * If set to false only the failed tests will be printed.
   */
  ResultVisitor(
    std::ostream & ostream,
    bool printGood = false
  )
  :
    OstreamVisitor(ostream),
    printGood(printGood)
  {
  }
  
  /** Method called when visiting a test.
   * @param test  The test to visit.
   */
  void operator()(TestBase & test) const
  {
    if(!printGood && !test.status)
      return;
    
    indent();
    ostream << test.name() << " ";
    switch(test.status)
    {
      case TestBase::eSuccess:
        ostream << "OK";
        break;
      case TestBase::eNotImplemented:
        ostream << "NOT IMPLEMENTED";
        break;
      case TestBase::eFailed:
        ostream << "FAILED";
        break;
      case TestBase::eError:
        ostream << "ERROR";
        break;
    }
    ostream << std::endl;
    if(test.status)
    {
      ++depth;
      if(!test.exception.message.empty())
      {
        indent();
        ostream << test.exception.message << std::endl;
      }
      if(!test.exception.details.empty())
      {
        indent();
        ostream << test.exception.details << std::endl;
      }
      if(!test.exception.file.empty())
      {
        indent();
        ostream << "file:" << test.exception.file << ", line:" << test.exception.line << std::endl;
      }
      --depth;
    }
  }
  
  /** Method called when visiting a test suite.
   * @param suite  The test suite to visit.
   */
  void operator()(TestSuite & suite) const
  {
    Counter counter;
    counter(suite);
    if(printGood || (counter.errors + counter.failures) > 0)
    {
      indent();
      ostream << suite.name() << std::endl;
    }
    
    ++depth;
    
    TestSuite::TestContainer::iterator it, end;
    for (
      it = suite.tests.begin(),
      end = suite.tests.end();
      it != end;
      ++it
    )
    {
      TestBase & test = *(*it);
      test(*this);
    } /* end of for */

    --depth;
  }
  
  /** Indicator to show only failed tests or all.
   */
  bool printGood;
};


/** Print an XML representation of the test results.
 * The generated XML is compatible to JUNIT, so that
 * continuous build environments like cruisecontrol
 * can use the test results without writing XLST transformations.
 */
struct JunitXmlResultVisitor
:
  public OstreamVisitor
{
  /** Create the xml visitor.
   * @param ostream    The stream to write to.
   * @param encoding   The encoding of the XML file.
   * The defaul encoding is UTF-8.
   */
  JunitXmlResultVisitor (
    std::ostream & ostream,
    std::string const & encoding = "UTF-8"
  )
  :
    OstreamVisitor(ostream),
    encoding(encoding)
  {
  
  }
  
  /** Method called when visiting a test.
   * @param test  The test to visit.
   */
  void operator()(TestBase & test) const
  {
    indent();
    ostream << "<testcase";
    ostream << " name=\"" << escape(test.name()) << "\"";
    ostream << " time=\"" << test.duration << "\"";
    ostream << (test.status ? ">" : "/>") << std::endl;
    ++depth;
    switch(test.status)
    {
      case TestBase::eFailed:
        indent();
        ostream << "<failure";
        break;
      case TestBase::eNotImplemented:
      case TestBase::eError:
        indent();
        ostream << "<error";
        break;
      default:
        break;
    }
    if(test.status)
    {
      std::string notImplementedMsg("Test has not been implemented (yet).");
      if(!test.exception.file.empty())
      {
        ostream << " file=\"" << escape(test.exception.file) << "\"";
        ostream << " line=\"" << test.exception.line << "\"";
      }
      if(test.status == TestBase::eNotImplemented)
      {
        ostream << " type=\"" << typeName(typeid(NotImplemented)) << "\"";
        ostream << " message=\"" << escape(notImplementedMsg) << "\"";
      }
      else
      {
        ostream << " type=\"" << test.error << "\"";
        ostream << " message=\"" << escape(test.exception.message) << "\"";
      }
      ostream << ">";
      
      if(test.status == TestBase::eNotImplemented)
      {
        ostream << escape(notImplementedMsg);
      }
      else
      {
        ostream << escape(test.exception.details);
      }
    }
    switch(test.status)
    {
      case TestBase::eFailed:
        ostream << "</failure>" << std::endl;
        break;
      case TestBase::eNotImplemented:
      case TestBase::eError:
        ostream << "</error>" << std::endl;
        break;
      default:
        break;
    }
    --depth;
    if(test.status)
    {
      indent();
      ostream << "</testcase>" << std::endl;
    }
  }
  
  /** Method called when visiting a test suite.
   * @param suite  The test suite to visit.
   */
  void operator()(TestSuite & suite) const
  {
    if(depth == 0)
    {
      ostream << "<?xml version=\"1.0\" encoding=\"" << encoding << "\"?>" << std::endl;
    }
    
    Counter counter;
    counter(suite);
    
    indent();
    ostream << "<testsuite";
    ostream << " name=\"" << escape(suite.name()) << "\"";
    ostream << " tests=\"" << counter.total << "\"";
    ostream << " failures=\"" << counter.failures << "\"";
    // junit does not have the notion of notimplemented tests
    // therefore we emulate that by setting the errors to the sum
    // and provide extra arguments to access the real numbers
    ostream << " errors=\"" << (counter.errors + counter.notImplemented)<< "\"";
    ostream << " errors_real=\"" << counter.failures << "\"";
    ostream << " errors_notimplemented=\"" << counter.notImplemented << "\"";
    ostream << " time=\"" << counter.duration << "\"";
    ostream << ">" << std::endl;
    
    ++depth;
    
    TestSuite::TestContainer::iterator it, end;
    for (
      it = suite.tests.begin(),
      end = suite.tests.end();
      it != end;
      ++it
    )
    {
      TestBase & test = *(*it);
      test(*this);
    } /* end of for */
    
    --depth;
    
    indent();
    ostream << "</testsuite>" << std::endl;
  }
  
  /** Escapes a string to conform to the XML standard.
   * @param text   The text to escape.
   * @return       The escaped text.
   */
  std::string escape(std::string const & text) const
  {
    std::string result = text;
    char const * replace[][2] = {
      {"&",  "&amp;"},
      {">",  "&gt;"},
      {"<",  "&lt;"},
      {"\"", "&quot;"},
      {"'",  "&apos;"}
    };
    
    for(int cnt = 0; cnt < 5; ++cnt)
    {
      std::string sought(replace[cnt][0]);
      std::string replacement(replace[cnt][1]);
      
      std::string::size_type found = 0;
      while((found = result.find(sought, found)) != std::string::npos)
      {
        result.replace(found, 1, replacement);
        ++found;
      }
    }
    return result;
  }
  
  /** The encoding of the XML file.
   */
  std::string encoding;
};


/** STL conform comparison functor.
 * This functor allows sorting test(suit)s by name.
 * By default all test(suit)s stored in a test suit are executed in the
 * order they were added to the suit. If the TestSuite::instance is used
 * via the \ref AutoRegister mechanism the orfer of the suites in the
 * TestSuite::instance are undefined.
 * To sort the test(suits) in the TestSuite::instance by name use the
 * following code:
 * \verbatim
    std::sort(
      globalTestSuite.tests.begin(),
      globalTestSuite.tests.end(),
      NameCompare()
    );
   \endverbatim
 */
struct NameCompare
:
  public std::binary_function <
    TestSuite::TestBasePtr,
    TestSuite::TestBasePtr,
    bool
  >
{
  /** The comparison method.
   */
  bool operator()(
    TestSuite::TestBasePtr const & left,
    TestSuite::TestBasePtr const & right
  )
  {
    return left->name() < right->name();
  }
};


/** Sorts tests by name.
 * If all test(suit)s should be sorted by name this visitor can be used.
 * This visitor uses \ref NameCompare to sort the test(suit)s by name.
 */
struct SortVisitor
:
  public Visitor
{
  /** Method called when visiting a test.
   */
  void operator()(TestBase &) const
  {
    //nop
  }
  
  /** Method called when visiting a test suite.
   */
  void operator()(TestSuite & suite) const
  {
    std::sort(
      suite.tests.begin(),
      suite.tests.end(),
      NameCompare()
    );

    TestSuite::TestContainer::iterator it, end;
    for (
      it = suite.tests.begin(),
      end = suite.tests.end();
      it != end;
      ++it
    )
    {
      TestBase & test = *(*it);
      test(*this);
    } /* end of for */
  }
};


/** Comparator used by \ref Assert.
 */
template < typename O >
struct Comparator
{
  /** Execute the comparison operation.
   * @param expected    The expected value.
   * @param actual      The actual value.
   */
  template < typename T >
  bool operator()(
    T const & expected,
    T const & actual
  )
  {
    O operation;
    return operation(expected, actual);
  }

  /** Execute the comparison operation with a delta.
   * @param expected    The expected value.
   * @param actual      The actual value.
   * @param delta       The actual delta.
   */
  bool operator()(
    double expected,
    double actual,
    double delta
  )
  {
    O operation;
    return operation(expected, actual, delta);
  }
  
  /** Generate the message from the given values or variable names.
   * @param expectedStr   The expected value or variable name.
   * @param actualStr     The actual value or variable name.
   */
  std::string message(
    char const * expectedStr,
    char const * actualStr
  )
  {
    std::ostringstream ss;
    ss << actualStr << " " << O::operation() << " ";
    ss << expectedStr << " FAILED.";
    return ss.str();
  }
  
  
  /** Generate the message from the given values or variable names.
   * @param expectedStr   The expected value or variable name.
   * @param actualStr     The actual value or variable name.
   * @param deltaStr      The delta value or variable name.
   */
  std::string message(
    char const * expectedStr,
    char const * actualStr,
    char const * deltaStr
  )
  {
    std::ostringstream ss;
    ss << actualStr << " " << O::operation() << " ";
    ss << expectedStr << "(" << deltaStr << ") FAILED.";
    return ss.str();
  }
  
  
  /** Generate the details from the given values.
   * @param expected      The expected value.
   * @param actual        The actual value.
   */
  template < typename T >
  std::string details(
    T const & expected,
    T const & actual  )
  {
    std::ostringstream ss;
    ss << actual << " " << O::operation() << " ";
    ss << expected << " FAILED.";
    return ss.str();
  }
  
  /** Generate the details from the given values.
   * @param expected    The expected value.
   * @param actual      The actual value.
   * @param delta       The actual delta.
   */
  template < typename T >
  std::string details(
    T const & expected,
    T const & actual,
    T const & delta  )
  {
    std::ostringstream ss;
    ss << actual << " " << O::operation() << " ";
    ss << expected << " (" << delta << ") FAILED.";
    return ss.str();
  }
    
  /** Generate the details from the given string.
   * This method is a specialization for std::string.
   * @param expected      The expected value.
   * @param actual        The actual value.
   */
  std::string details(
    std::string const & expected,
    std::string const & actual
  )
  {
    int actualsize = actual.size();
    int expectedsize = expected.size();
    
    std::ostringstream ss;
    ss << "\"" << actual;
    ss << "\" (" << actualsize << ")";
    ss << " " << O::operation() << " ";
    ss << "\"" << expected;
    ss << "\" (" << expectedsize << ")";
    ss << " FAILED.";
    return ss.str();
  }
  
  /** Generate the details from the given char.
   * This method is a specialization for char.
   * @param expected      The expected value.
   * @param actual        The actual value.
   */
  std::string details(
    char const expected,
    char const actual
  )
  {
    int actualint = actual;
    int expectedint = expected;
    
    std::ostringstream ss;
    ss << actual;
    ss << "(0x" << std::hex << actualint;
    ss << "," << std::dec << actualint << ")";
    ss << " " << O::operation() << " ";
    ss << expected;
    ss << "(0x" << std::hex << expectedint;
    ss << "," << std::dec << expectedint << ")";
    ss << " FAILED.";
    return ss.str();
  }
  
  /** Generate the details from the given unsigned char.
   * This method is a specialization for unsigned char.
   * @param expected      The expected value.
   * @param actual        The actual value.
   */
  std::string details(
    unsigned char const expected,
    unsigned char const actual  )
  {
    int actualint = actual;
    int expectedint = expected;
    
    std::ostringstream ss;
    ss << actual;
    ss << "(0x" << std::hex << actualint;
    ss << "," << std::dec << actualint << ")";
    ss << " " << O::operation() << " ";
    ss << expected;
    ss << "(0x" << std::hex << expectedint;
    ss << "," << std::dec << expectedint << ")";
    ss << " FAILED.";
    return ss.str();
  }
  
  /** Generate the details from the given signed char.
   * This method is a specialization for signed char.
   * @param expected      The expected value.
   * @param actual        The actual value.
   */
  std::string details(
    signed char const expected,
    signed char const actual  )
  {
    int actualint = actual;
    int expectedint = expected;
    
    std::ostringstream ss;
    ss << actual;
    ss << "(0x" << std::hex << actualint;
    ss << "," << std::dec << actualint << ")";
    ss << " " << O::operation() << " ";
    ss << expected;
    ss << "(0x" << std::hex << expectedint;
    ss << "," << std::dec << expectedint << ")";
    ss << " FAILED.";
    return ss.str();
  }
};


/** \ref Comparator trait for equal.
 */
struct Equal
{
  /** Execute the comparison operation.
   */
  template < typename T >
  bool operator()(
    T const & expected,
    T const & actual
  )
  {
    return actual == expected;
  }
  
  /** Execute the comparison operation with a delta.
   */
  bool operator()(
    double expected,
    double actual,
    double delta
  )
  {
    return std::fabs(expected - actual) <= delta;
  }
  
  /** Textual representation of the operation.
   * @return The textual representation of the operation.
   */
  static char const * operation()
  {
    return "==";
  }
};


/** \ref Comparator trait for not equal.
 */
struct NotEqual
{
  /** Execute the comparison operation.
   */
  template < typename T >
  bool operator()(
    T const & expected,
    T const & actual
  )
  {
    // actual != expected;
    return !(actual == expected);
  }
  
  /** Execute the comparison operation with a delta.
   */
  bool operator()(
    double expected,
    double actual,
    double delta
  )
  {
    return std::fabs(expected - actual) > delta;
  }
  
  /** Textual representation of the operation.
   * @return The textual representation of the operation.
   */
  static char const * operation()
  {
    return "!=";
  }
};


/** \ref Comparator trait for less.
 */
struct Less
{
  /** Execute the comparison operation.
   */
  template < typename T >
  bool operator()(
    T const & expected,
    T const & actual
  )
  {
    return actual < expected;
  }
  
  /** Execute the comparison operation with a delta.
   */
  bool operator()(
    double expected,
    double actual,
    double delta
  )
  {
    return (expected - actual) > 0.0 && std::fabs(expected - actual) > delta;
  }
  
  /** Textual representation of the operation.
   * @return The textual representation of the operation.
   */
  static char const * operation()
  {
    return "<";
  }
};


/** \ref Comparator trait for less or equal.
 */
struct LessEqual
{
  /** Execute the comparison operation.
   */
  template < typename T >
  bool operator()(
    T const & expected,
    T const & actual
  )
  {
    // actual <= expected;
    return ((actual < expected) || (actual == expected));
  }
  
  /** Execute the comparison operation with a delta.
   */
  bool operator()(
    double expected,
    double actual,
    double delta
  )
  {
    return Less()(expected, actual, delta) || Equal()(expected, actual, delta);
  }
  
  /** Textual representation of the operation.
   * @return The textual representation of the operation.
   */
  static char const * operation()
  {
    return "<=";
  }
};


/** \ref Comparator trait for greater.
 */
struct Greater
{
  /** Execute the comparison operation.
   */
  template < typename T >
  bool operator()(
    T const & expected,
    T const & actual
  )
  {
    // actual > expected;
    return expected < actual;
  }
  
  /** Execute the comparison operation with a delta.
   */
  bool operator()(
    double expected,
    double actual,
    double delta
  )
  {
    return (expected - actual) < 0.0 && std::fabs(expected - actual) > delta;
  }
  
  /** Textual representation of the operation.
   * @return The textual representation of the operation.
   */
  static char const * operation()
  {
    return ">";
  }
};


/** \ref Comparator trait for greater or equal.
 */
struct GreaterEqual
{
  /** Execute the comparison operation.
   */
  template < typename T >
  bool operator()(
    T const & expected,
    T const & actual
  )
  {
    // actual >= expected;
    return ((expected < actual) || (expected == actual));
  }
  
  /** Execute the comparison operation with a delta.
   */
  bool operator()(
    double expected,
    double actual,
    double delta
  )
  {
    return Greater()(expected, actual, delta) || Equal()(expected, actual, delta);
  }

  /** Textual representation of the operation.
   * @return The textual representation of the operation.
   */
  static char const * operation()
  {
    return ">=";
  }
};


/** The asserter.
 * This class checks if an assertation is true and if false throws
 * an exception.
 * This class is used by the UNITTEST_ASSERT and
 * UNITTEST_ASSERT_DELTA macros.
 */
template < typename C >
struct Assert
{
  /** Create the asserter.
   */
  template < typename T >
  Assert(
    T const & expected,
    T const & actual,
    char const * const expectedStr,
    char const * const actualStr,
    char const * const file,
    long line
  )
  :
    added(false),
    failed(false),
    ex(
      file,
      line
    )
  {
    Comparator < C > comparator;
    failed = !comparator(expected, actual);
    if(failed)
    {
      ex.message = comparator.message(
        expectedStr,
        actualStr
      );
      ex.details = comparator.details(
        expected,
        actual
      );
    }
  }
  
  /** Create the asserter.
   */
  template < typename T >
  Assert(
    T const & expected,
    T const & actual,
    T const & delta,
    char const * const expectedStr,
    char const * const actualStr,
    char const * const deltaStr,
    char const * const file,
    long line
  )
  :
    added(false),
    failed(false),
    ex(
      file,
      line
    )
  {
    Comparator < C > comparator;
    failed = !comparator(expected, actual, delta);
    if(failed)
    {
      ex.message = comparator.message(
        expectedStr,
        actualStr,
        deltaStr
      );
      ex.details = comparator.details(
        expected,
        actual,
        delta
      );
    }
  }
  
  /** Check the assertation.
   * The design of this beauty was inpired by the Enforce class
   * by Andrei Alexandrescu.
   */
  void operator*() const
  {
    if(failed)
    {
      throw Exception(ex);
    }
  }
  
  /** Enrich the details of the assertation.
   * @param msg   The detail to add.
   * The design of this beauty was inpired by the Enforce class
   * by Andrei Alexandrescu.
   */
  template < typename MsgType >
  Assert & operator()(const MsgType& msg)
  {
    if(!added)
    {
      added = true;
      ex.details += " ";
    }
    if(failed)
    {
      // Here we have time; no need to be super-efficient
      std::ostringstream ss;
      ss << msg;
      ex.details += ss.str();
    }
    
    return *this;
  }
  
  /** Flag that indicated that we have added user defined
   * content to the details.
   */
  bool added;
  /** Flag to indicate that the comparison failed.
   */
  bool failed;
  /** The \ref Exception.
   */
  Exception ex;
};

/** \def UNITTEST_ASSERT(_C_, _E_, _A_)
 * Create an assertation.
 * Examples (e is the expected value, a is the actual value):
 * \verbatim
     UNITTEST_ASSERT(Equal, e, a);
     UNITTEST_ASSERT(NotEqual, e, a);
     UNITTEST_ASSERT(Less, e, a);
     UNITTEST_ASSERT(LessEqual, e, a);
     UNITTEST_ASSERT(Greater, e, a);
     UNITTEST_ASSERT(GreaterEqual, e, a);
   \endverbatim
 */
#define UNITTEST_ASSERT(_C_, _E_, _A_) \
  *unittest::Assert < _C_ > ( \
    _E_, \
    _A_, \
    #_E_, \
    #_A_, \
    __FILE__, \
    __LINE__ \
  )


/** \def UNITTEST_ASSERT(_C_, _E_, _A_, _D_)
 * Create an assertation.
 * Examples (e is the expected value, a is the actual value, d is the delta):
 * \verbatim
     UNITTEST_ASSERT_DELTA(Equal, e, a, d);
     UNITTEST_ASSERT_DELTA(NotEqual, e, a, d);
     UNITTEST_ASSERT_DELTA(Less, e, a, d);
     UNITTEST_ASSERT_DELTA(LessEqual, e, a, d);
     UNITTEST_ASSERT_DELTA(Greater, e, a, d);
     UNITTEST_ASSERT_DELTA(GreaterEqual, e, a, d);
   \endverbatim
 */
#define UNITTEST_ASSERT_DELTA(_C_, _E_, _A_, _D_) \
  *unittest::Assert < _C_ > ( \
    _E_, \
    _A_, \
    _D_, \
    #_E_, \
    #_A_, \
    #_D_, \
    __FILE__, \
    __LINE__ \
  )

/** \def UNITTEST_EXCEPTION(_E_, _C_)
 * Assert the throwing of a specific Exception.
 * Examples (e is the expected exception, c is the c++ code to execute):
 * \verbatim
     UNITTEST_EXCEPTION(
       std::logic_error,
       somecode;
     )
   \endverbatim
 */
#define UNITTEST_EXCEPTION(_E_, _C_) \
  try \
  { \
    _C_ \
    throw unittest::Exception( \
      __FILE__, \
      __LINE__, \
      "Expected exception " #_E_ " to be thrown.", \
      "No exception was thrown." \
    ); \
  } \
  catch(unittest::Exception & ex) \
  { \
    throw; \
  } \
  catch(_E_ const & ex) \
  { \
    /* nop */ \
  } \
  catch(std::exception const & ex) \
  { \
    std::ostringstream ss; \
    ss << "Expected exception " #_E_ " to be thrown."; \
    std::string msg = ss.str(); \
    ss.str(""); \
    ss << "Caught " << unittest::typeName(typeid(ex)) << " instead"; \
    ss << " (" << ex.what() << ")."; \
    throw unittest::Exception( \
      __FILE__, \
      __LINE__, \
      msg, \
      ss.str() \
    ); \
  } \
  catch(...) \
  { \
    std::ostringstream ss; \
    ss << "Expected exception " #_E_ " to be thrown."; \
    std::string msg = ss.str(); \
    ss.str(""); \
    ss << "Caught an unknown C++ exception instead"; \
    throw unittest::Exception( \
      __FILE__, \
      __LINE__, \
      msg, \
      ss.str() \
    ); \
  }


/** \def UNITTEST_NO_EXCEPTION(_C_)
 * Assert the nonthowing of a axceptions.
 * Examples (c is the c++ code to execute):
 * \verbatim
     UNITTEST_NO_EXCEPTION(
       somecode;
     )
   \endverbatim
 */
#define UNITTEST_NO_EXCEPTION(_C_) \
  try \
  { \
    _C_ \
  } \
  catch(unittest::Exception & ex) \
  { \
    throw; \
  } \
  catch(std::exception const & ex) \
  { \
    std::ostringstream ss; \
    ss << "Expected no exception to be thrown."; \
    std::string msg = ss.str(); \
    ss.str(""); \
    ss << "Caught " << unittest::typeName(typeid(ex)) << " instead"; \
    ss << " (" << ex.what() << ")."; \
    throw unittest::Exception( \
      __FILE__, \
      __LINE__, \
      msg, \
      ss.str() \
    ); \
  } \
  catch(...) \
  { \
    std::ostringstream ss; \
    ss << "Expected no exception to be thrown."; \
    std::string msg = ss.str(); \
    ss.str(""); \
    ss << "Caught an unknown C++ exception instead"; \
    throw unittest::Exception( \
      __FILE__, \
      __LINE__, \
      msg, \
      ss.str() \
    ); \
  }


} /* namespace unittest */


namespace std
{

/** ostream operator to print an Exception.
 */
template < class charT, class traits >
std::basic_ostream < charT, traits > & operator << (
  std::basic_ostream < charT, traits > & ostream,
  unittest::Exception const & ex
)
{
  ostream << ex.message << std::endl;
  if(!ex.details.empty())
  {
    ostream << ex.details << std::endl;
  }
  if(!ex.file.empty())
  {
    ostream << "file:" << ex.file << ", line:" << ex.line << std::endl;
  }
  return ostream;
}


} /* end of namespace std */


#endif /*unittest_unittest_h*/
